
// includes
#include "../platform/windows.hpp"

#include "../platform/Window.hpp"
#include "Mouse.hpp"

using namespace bleifrei::platform;

namespace bleifrei {

	input::Mouse Singleton<input::Mouse>::instance;

	namespace input {

		bool
		Mouse::init(void)
		{
			m_window = 0;
			m_mode = MouseModeAbsolute;
			m_screenX = m_x = 0;
			m_screenY = m_y = 0;
			m_moveX = 0;
			m_moveY = 0;

			return true;
		}

		void
		Mouse::destroy(void)
		{
		}

		void
		Mouse::setMode(MouseMode mode)
		{
			if (mode == MouseModeRelative) {
				int halfWidth, halfHeight;
				if (m_window)
				{
					halfWidth = m_window->getWidth() / 2;
					halfHeight = m_window->getHeight() / 2;
				}
				else
				{
					halfWidth = 256;
					halfHeight = 256;
				}

				SetCursorPos(halfWidth, halfHeight);
			}
			m_mode = mode;
		}

		void
		Mouse::setScreenPosition(int x, int y)
		{
			SetCursorPos(x, y);
			m_screenX = m_x = x;
			m_screenY = m_y = y;

			Window::getInstance().toLocal(m_x, m_y);
		}

		bool
		Mouse::isDown(int button)
		{
			return (GetAsyncKeyState(button) & 0x8000) != 0;
		}

		void
		Mouse::update(void)
		{
			POINT mouse_pos;
			GetCursorPos(&mouse_pos);

			m_screenX = m_x = mouse_pos.x;
			m_screenY = m_y = mouse_pos.y;

			m_moveZ = m_wheelDelta;
			m_wheelDelta = 0;

			if (m_mode == MouseModeRelative) {
				int halfWidth = 256;
				int halfHeight = 256;
				if (m_window)
				{
					halfWidth = m_window->getWidth() / 2;
					halfHeight = m_window->getHeight() / 2;
				}

				SetCursorPos(halfWidth, halfHeight);
				m_moveX = m_x - halfWidth;
				m_moveY = m_y - halfHeight;
			}

			Window::getInstance().toLocal(m_x, m_y);
		}

		void
		Mouse::scrollWheel(int delta)
		{
			m_wheelDelta += delta;
		}
	}
}