
#ifndef _include_bleifrei_platform_Mouse_hpp_
#define _include_bleifrei_platform_Mouse_hpp_ 


// includes
#include "../Singleton.hpp"

namespace bleifrei {
	namespace platform {
		class Window;
	}
	namespace input {

		enum MouseButton {
			MouseButtonLeft = 0x01,
			MouseButtonRight = 0x02,
			MouseButtonMiddle = 0x04,
			MouseButtonPrevious = 0x05,
			MouseButtonNext = 0x06
		};

		enum MouseMode {
			MouseModeRelative,
			MouseModeAbsolute
		};

		class MouseEventListener {
		};

		class Mouse : public Singleton<Mouse>
		{
			friend class Singleton<Mouse>;
		public:
			bool init(void);
			void destroy(void);

			void setMode(MouseMode mode);
			void setScreenPosition(int x, int y);

			void getMovement(int &x, int &y)								{ x = m_moveX; y = m_moveY;			}
			int getMovementX(void)											{ return m_moveX;					}
			int getMovementY(void)											{ return m_moveY;					}
			int getMovementZ(void)											{ return m_moveZ;					}

			void getPosition(int &x, int &y)								{ x = m_x; y = m_y;					}
			int getPositionX(void)											{ return m_x;						}
			int getPositionY(void)											{ return m_y;						}

			void getScreenPosition(int &x, int &y)							{ x = m_screenX; y = m_screenY;		}
			int getScreenPositionX(void)									{ return m_screenX;					}
			int getScreenPositionY(void)									{ return m_screenY;					}

			void attachToWindow(platform::Window *window)					{ m_window = window;				}

			bool isDown(int button);

			void update(void);

			void scrollWheel(int delta);

		private:
			Mouse(void) {}
			~Mouse(void) {}

			MouseMode m_mode;
			int m_x, m_y;
			int m_screenX, m_screenY;
			int m_moveX, m_moveY, m_moveZ;
			int m_wheelDelta;
			platform::Window *m_window;
		};

	}
}


#endif //_include_bleifrei_platform_Mouse_hpp_
